require "import"
-- استيراد حزم واجهة المستخدم لنظام أندرويد
import "android.widget.*"
-- استيراد حزمة AndroLua
import "com.androlua.*"
-- استيراد فئة View (عرض)
import "android.view.View"
-- استيراد فئة Environment (البيئة) للوصول إلى التخزين الخارجي
import "android.os.Environment"
-- استيراد فئة URLEncoder لتشفير عناوين URL
import "java.net.URLEncoder"
-- استيراد فئة File للتعامل مع الملفات
import "java.io.File"
-- استيراد فئة FileOutputStream للكتابة إلى الملفات
import "java.io.FileOutputStream"
-- استيراد فئة BitmapFactory لفك تشفير الصور النقطية
import "android.graphics.BitmapFactory"

-- دالة للحصول على اسم ملف فريد (صورة AI)
function getUniqueFilename(name)
  local t = os.date("*t") -- الحصول على الوقت الحالي كجدول
  -- تنسيق الطابع الزمني لضمان التفرد
  local ts = string.format("%04d%02d%02d_%02d%02d%02d", t.year, t.month, t.day, t.hour, t.min, t.sec)
  -- بناء المسار الكامل للملف في التخزين الخارجي
  return Environment.getExternalStorageDirectory().toString() .. "/" .. name .. "_" .. ts .. ".png"
end

-- دالة لتنزيل الصورة من واجهة برمجة تطبيقات AI
function downloadImage(text, callback)
  -- تشفير النص للاستخدام في عنوان URL
  local encoded = URLEncoder.encode(text, "UTF-8")
  -- بناء عنوان URL لواجهة برمجة تطبيقات مولد الفن بالذكاء الاصطناعي
  local url = "https://sheikhhridoy.nagad.my.id/api/ai-art-generator.php?prompt=" .. encoded
  -- الحصول على مسار فريد لحفظ الصورة
  local path = getUniqueFilename("ai_image") -- اسم الملف الأساسي "صورة_ذكاء_اصطناعي"
  -- تنفيذ طلب HTTP GET لتنزيل الصورة
  Http.get(url, nil, function(code, content)
    if code == 200 and content then
      -- فتح ملف للكتابة الثنائية
      local f = io.open(path, "wb")
      if f then
        f:write(content) -- كتابة محتوى الصورة إلى الملف
        f:close() -- إغلاق الملف
        callback(true, path) -- استدعاء رد الاتصال بنجاح ومسار الملف
      else
        callback(false, "فشل الكتابة.") -- استدعاء رد الاتصال بفشل الكتابة
      end
    else
      callback(false, "خطأ: " .. code) -- استدعاء رد الاتصال برمز الخطأ
    end
  end)
end

-- تعريف تخطيط واجهة المستخدم (UI)
layout = {
  LinearLayout, -- حاوية تخطيط عمودي
  orientation = "vertical", -- اتجاه عمودي
  layout_width = "fill", -- عرض يملأ الأب
  layout_height = "fill", -- ارتفاع يملأ الأب
  padding = "16dp", -- حشوة داخلية 16dp
  {
    EditText, -- مربع إدخال النص
    id = "inputText", -- معرف فريد لمربع النص
    hint = "أدخل الوصف", -- تلميح يظهر للمستخدم
    layout_width = "fill", -- عرض يملأ الأب
    layout_height = "wrap_content", -- ارتفاع يحتوى المحتوى
  },
  {
    Button, -- زر توليد الصورة
    id = "generateButton", -- معرف فريد للزر
    text = "توليد صورة", -- نص الزر
    layout_width = "fill", -- عرض يملأ الأب
    layout_height = "wrap_content", -- ارتفاع يحتوى المحتوى
  },
  {
    TextView, -- مربع نص لعرض النتائج أو الرسائل
    id = "resultText", -- معرف فريد لمربع النص
    layout_width = "fill", -- عرض يملأ الأب
    layout_height = "wrap_content", -- ارتفاع يحتوى المحتوى
  },
  {
    ImageView, -- لعرض الصورة المولدة
    id = "generatedImage", -- معرف فريد لمربع الصورة
    layout_width = "fill", -- عرض يملأ الأب
    layout_height = "300dp", -- ارتفاع ثابت 300dp
    visibility = View.GONE, -- مخفية في البداية
  },
  {
    Button, -- زر الخروج
    id = "exitButton", -- معرف فريد لزر الخروج
    text = "خروج", -- نص الزر
    layout_width = "wrap_content", -- عرض يحتوى المحتوى
    layout_height = "wrap_content", -- ارتفاع يحتوى المحتوى
  },
  {
    TextView, -- نص يظهر رابط قناة Telegram
    text = "https://t.me/+dJrRrA6wxNlmZmU0", -- رابط القناة
    layout_width = "fill", -- عرض يملأ الأب
    layout_height = "wrap_content", -- ارتفاع يحتوى المحتوى
    gravity = "center", -- محاذاة النص في المنتصف
    padding = "8dp", -- حشوة داخلية 8dp
  }
}

-- إنشاء مربع حوار Lua
dlg = LuaDialog(this)
dlg.setTitle("مولد صور بالذكاء الاصطناعي معربة من أخوكم السايح فتحي") -- تعيين عنوان لمربع الحوار
dlg.setView(loadlayout(layout)) -- تحميل التخطيط في مربع الحوار

-- دالة لتحديث نص النتيجة
function updateResult(t)
  resultText.text = t or ""
end

-- معالج النقر لزر "توليد صورة"
generateButton.onClick = function()
  local prompt = inputText.text -- الحصول على النص المدخل من المستخدم
  if prompt and #prompt > 0 then
    updateResult("جاري التوليد...") -- عرض رسالة "جاري التوليد"
    generatedImage.visibility = View.GONE -- إخفاء الصورة السابقة
    downloadImage(prompt, function(ok, result) -- استدعاء دالة تنزيل الصورة
      if ok then
        local bmp = BitmapFactory.decodeFile(result) -- فك تشفير الصورة من الملف
        if bmp then
          generatedImage.setImageBitmap(bmp) -- تعيين الصورة المولدة في ImageView
          generatedImage.visibility = View.VISIBLE -- جعل الصورة مرئية
          updateResult("حفظ في:\n" .. result) -- عرض رسالة النجاح ومسار الحفظ
        else
          updateResult("فشل فك التشفير.") -- عرض رسالة فشل فك التشفير
        end
      else
        updateResult(result) -- عرض رسالة الخطأ
      end
    end)
  else
    updateResult("أدخل الوصف.") -- مطالبة المستخدم بإدخال وصف
  end
end

-- معالج النقر لزر "خروج"
exitButton.onClick = function()
  dlg.dismiss() -- إغلاق مربع الحوار
end

-- عرض مربع الحوار
dlg.show()